package com.huimv.farm.musk.breed.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.breed.entity.BreedInspect;
import com.huimv.farm.musk.breed.entity.BreedMedicine;
import com.huimv.farm.musk.breed.mapper.BreedInspectMapper;
import com.huimv.farm.musk.breed.service.IBreedInspectService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.Map;

/**
 * <p>
 * 日常巡检 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class BreedInspectServiceImpl extends ServiceImpl<BreedInspectMapper, BreedInspect> implements IBreedInspectService {

    @Autowired
    private BreedInspectMapper inspectMapper;

    @Override
    public Result listPage(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String pageNum = paramsMap.get("pageNum");
        String pageSize = paramsMap.get("pageSize");
        String muskId = paramsMap.get("muskId");
        if ("".equals(pageNum) || null == pageNum) {
            pageNum = "1";
            pageSize = "20";
        }
        QueryWrapper<BreedInspect> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).like(StringUtils.isNotBlank(muskId), "musk_id", muskId);
        Page page = new Page<BreedInspect>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS, inspectMapper.selectPage(page, queryWrapper));
    }

    @Override
    public Result add(BreedInspect inspect) {
        inspect.setCreateTime(new Date());
        return new Result(ResultCode.SUCCESS, inspectMapper.insert(inspect));
    }

    @Override
    public Result edit(BreedInspect inspect) {
        return new Result(ResultCode.SUCCESS, inspectMapper.updateById(inspect));
    }

    @Override
    public Result remove(Map<String, String> paramsMap) {
        String ids = paramsMap.get("ids");
        String[] split = ids.split(",");
        for (String s : split) {
            inspectMapper.deleteById(s);
        }
        return new Result(ResultCode.SUCCESS);
    }
}
