package com.huimv.farm.musk.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.mapper.BreedMuskInfoMapper;
import com.huimv.farm.musk.breed.mapper.MuskEarmarkMapper;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.common.utils.ResultUtil;
import com.huimv.farm.musk.entity.BaseBuilding;
import com.huimv.farm.musk.entity.vo.TreeBasePigpen;
import com.huimv.farm.musk.entity.vo.TreeBasePigpen2;
import com.huimv.farm.musk.mapper.BaseBuildingMapper;
import com.huimv.farm.musk.service.IBaseBuildingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-08-15
 */
@Service
public class BaseBuildingServiceImpl extends ServiceImpl<BaseBuildingMapper, BaseBuilding> implements IBaseBuildingService {

    @Autowired
    private BaseBuildingMapper baseBuildingMapper;
    @Autowired
    private BreedMuskInfoMapper muskInfoMapper;
    @Autowired
    private MuskEarmarkMapper earmarkMapper;


    @Override
    @Transactional
    public Result addBuilding(Map<String,String> map) {
        String farmId = map.get("farmId");
        String number = map.get("number");
        String buildName = map.get("buildName");
        if (this.count(new QueryWrapper<BaseBuilding>().eq("build_name", buildName)) != 0) {
            return new Result(10001, "栋舍名已存在", false);
        }
        BaseBuilding baseBuilding = new BaseBuilding();
        baseBuilding.setBuildName(buildName);
        baseBuilding.setParentId(0);
        baseBuilding.setFType(1);
        baseBuilding.setFarmId(Integer.parseInt(farmId));
        this.save(baseBuilding);
        BaseBuilding baseBuilding1 = baseBuildingMapper.selectOne(new QueryWrapper<BaseBuilding>().eq("farm_id", farmId)
                .eq("build_name", buildName).eq("f_type",1));

        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
        Integer parentId = baseBuilding1.getId();
        queryWrapper.eq("farm_id", farmId).eq("parent_id", parentId).orderByDesc("id").last(" limit 1");
        BaseBuilding building = baseBuildingMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(building)) {
            for (int i = 1; i < Integer.parseInt(number) + 1; i++) {
                BaseBuilding baseBuilding2 = new BaseBuilding();
                baseBuilding2.setBuildName(i + "号栏");
                baseBuilding2.setOther2(String.valueOf(i));
                baseBuilding2.setParentId(parentId);
                baseBuilding2.setFarmId(Integer.parseInt(farmId));
                baseBuilding2.setFType(2);
                baseBuildingMapper.insert(baseBuilding2);
            }
        } else {
            String other2 = building.getOther2();
            for (int i =  1; i <Integer.parseInt(number) + 1; i++) {
                BaseBuilding baseBuilding2 = new BaseBuilding();
                baseBuilding2.setBuildName(Integer.parseInt(other2) + i + "号栏");
                baseBuilding2.setOther2(String.valueOf(Integer.parseInt(other2) + i));
                baseBuilding2.setParentId(parentId);
                baseBuilding2.setFarmId(Integer.parseInt(farmId));
                baseBuilding2.setFType(2);
                baseBuildingMapper.insert(baseBuilding);
            }
        }

        return Result.SUCCESS();
    }

    @Override
    @Transactional
    public Result updateBuilding(Map<String,String> map) {
        String farmId = map.get("farmId");
        String number = map.get("number");
        String buildName = map.get("buildName");
        String id = map.get("id");
        if (this.count(new QueryWrapper<BaseBuilding>().eq("build_name", buildName).ne("id", id)) != 0) {
            return new Result(10001, "栋舍名已存在", false);
        }
        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", id);
        List<BaseBuilding> buildings = baseBuildingMapper.selectList(queryWrapper);
        if (buildings.size() != 0) {
            List<Integer> idList = new ArrayList<>();
            buildings.forEach(i->{
                idList.add(i.getId());
            });
            QueryWrapper<BreedMuskInfo> queryWrapper1 = new QueryWrapper<>();
            queryWrapper1.in("unit_id",idList).eq("is_leave",0);
            List<BreedMuskInfo> muskInfos = muskInfoMapper.selectList(queryWrapper1);
            if (muskInfos.size() != 0) {
                return new Result(10001, "栏位中存在林麝，请先转舍后再进行栋舍栏位修改", false);
            }
            BaseBuilding building = this.getById(id);
            building.setBuildName(buildName);
            this.updateById(building);
            this.removeByIds(idList);
            for (int i = 1; i < Integer.parseInt(number) + 1; i++) {
                BaseBuilding baseBuilding = new BaseBuilding();
                baseBuilding.setBuildName(i + "号栏");
                baseBuilding.setOther2(String.valueOf(i));
                baseBuilding.setParentId(Integer.parseInt(id));
                baseBuilding.setFarmId(Integer.parseInt(farmId));
                baseBuilding.setFType(2);
                baseBuildingMapper.insert(baseBuilding);
            }
        }else {
            for (int i = 1; i < Integer.parseInt(number) + 1; i++) {
                BaseBuilding baseBuilding = new BaseBuilding();
                baseBuilding.setBuildName(i + "号栏");
                baseBuilding.setOther2(String.valueOf(i));
                baseBuilding.setParentId(Integer.parseInt(id));
                baseBuilding.setFarmId(Integer.parseInt(farmId));
                baseBuilding.setFType(2);
                baseBuildingMapper.insert(baseBuilding);
            }
        }
        //TODO:后面补充羊只和设备的名称修改
        return ResultUtil.updateResult(1);
    }

    @Override
    @Transactional
    public Result deleteBuilding(Map<String, Integer> map) {
        Integer id = map.get("id");
        //TODO:后面再进行补充羊只和设备情况
        QueryWrapper<BreedMuskInfo> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("unit_id", id).eq("is_leave", 0);
        Integer count = muskInfoMapper.selectCount(queryWrapper1);
        if (count != 0) {
            return new Result(10001,"删除失败，该栋舍存在林麝",false);
        }
//        if (ObjectUtil.isNotEmpty(cageIds)){
//            if (ObjectUtil.isNotEmpty(baseDuckInfoMapper.selectList(new QueryWrapper<BaseDuckInfo>().in("unit_id",cageIds)))) {
//                return new Result(10001,"删除失败，该栋舍下属笼中存在鸭只",false);
//            }
//            if (ObjectUtil.isNotEmpty(envDeviceMapper.selectList(new QueryWrapper<EnvDevice>().in("unit_id",cageIds)))) {
//                return new Result(10001,"删除失败，该栋舍下属笼中存在设备",false);
//            }
//        }
//        if (ObjectUtil.isNotEmpty(envDeviceMapper.selectList(new QueryWrapper<EnvDevice>().eq("unit_id",id)))) {
//            return new Result(10001,"删除失败，该栋舍存在设备",false);
//        }
        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("parent_id", id);
        baseBuildingMapper.delete(queryWrapper);
        this.removeById(id);
        return new Result(10000, "删除成功", true);

    }

    @Override
    public Result listBuilding(Map<String, String> map) {
        String farmId = map.get("farmId");
        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<BaseBuilding>()
                .eq("farm_id", farmId)
                .eq("f_type", 1);
        return new Result(ResultCode.SUCCESS, baseBuildingMapper.selectList(queryWrapper));
    }

    @Override
    public Result listTreeBuilding(Map<String, String> map) {
        String farmId = map.get("farmId");
        List<BaseBuilding> baseBuildings = this.list(new QueryWrapper<BaseBuilding>()
                .eq("farm_id", farmId)
                .orderByAsc("id"));
        List<TreeBasePigpen> treeBasePigpens = parseBizBaseArea(baseBuildings);
        return new Result(ResultCode.SUCCESS, treeBasePigpens);
    }

    @Override
    public List<BaseBuilding> listUnit() {
        return this.list(new QueryWrapper<BaseBuilding>().eq("f_type",1));
    }

    @Override
    public Result listTreeBuilding2(Map<String, String> map) {
        String farmId = map.get("farmId");
        List<BaseBuilding> baseBuildings = this.list(new QueryWrapper<BaseBuilding>()
                .eq("farm_id", farmId)
                .orderByAsc("id"));
        List<TreeBasePigpen2> treeBasePigpens = parseBizBaseArea2(baseBuildings);
        return new Result(ResultCode.SUCCESS, treeBasePigpens);
    }

    /**
     * 查询结果 转换成树形结构
     *
     * @param bizBaseAreas 原始数据
     * @return 树
     */
    private List<TreeBasePigpen> parseBizBaseArea(List<BaseBuilding> bizBaseAreas) {
        //构建需要展示的树形节点结构
        Map<Object, TreeBasePigpen> nodeMap = bizBaseAreas.stream().map(basePigpen -> {
            TreeBasePigpen baseVo = new TreeBasePigpen();
            baseVo.setId(basePigpen.getId());
            baseVo.setFarmId(basePigpen.getFarmId());
            baseVo.setParentId(basePigpen.getParentId());
            baseVo.setBuildName(basePigpen.getBuildName());
            if (basePigpen.getFType() == 2 && basePigpen.getDisabled() == 1) {
                baseVo.setDisabled(true);
            } else {
                baseVo.setDisabled(false);
            }
            baseVo.setChildNode(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(TreeBasePigpen::getId, b -> b, (k1, k2) -> k1));
        //创建数组存父亲节点
        ArrayList<TreeBasePigpen> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item -> {
            Integer parentCode = item.getParentId();
            if (nodeMap.get(parentCode) == null) {
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            } else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getChildNode().add(item);
            }
        });
        return roots;
    }


    private List<TreeBasePigpen2> parseBizBaseArea2(List<BaseBuilding> bizBaseAreas) {
        //构建需要展示的树形节点结构
        Map<Object, TreeBasePigpen2> nodeMap = bizBaseAreas.stream().map(basePigpen -> {
            TreeBasePigpen2 baseVo = new TreeBasePigpen2();
            baseVo.setId(basePigpen.getId());
            baseVo.setFarmId(basePigpen.getFarmId());
            baseVo.setParentId(basePigpen.getParentId());
            baseVo.setBuildName(basePigpen.getBuildName());
            baseVo.setChildNode(new ArrayList<>());
            return baseVo;
        }).collect(Collectors.toMap(TreeBasePigpen2::getId, b -> b, (k1, k2) -> k1));
        //创建数组存父亲节点
        ArrayList<TreeBasePigpen2> roots = new ArrayList<>();
        //构建树形
        nodeMap.values().forEach(item -> {
            Integer parentCode = item.getParentId();
            if (nodeMap.get(parentCode) == null) {
                //父节点为空，说明当前节点就已经是父节点了 将该节点存起来
                roots.add(item);
            } else {
                //父节点 不为空，说明有父节点，拿到该节点的父节点的孩子节点（就是我自己),将该节点存起来
                nodeMap.get(parentCode).getChildNode().add(item);
            }
        });
        return roots;
    }
}
