package com.huimv.farm.musk.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.musk.common.utils.DataUill;
import com.huimv.farm.musk.common.utils.Result;
import com.huimv.farm.musk.common.utils.ResultCode;
import com.huimv.farm.musk.entity.BaseBuilding;
import com.huimv.farm.musk.entity.DevEnergy;
import com.huimv.farm.musk.entity.DevEnv;
import com.huimv.farm.musk.entity.SysCameraArea;
import com.huimv.farm.musk.mapper.BaseBuildingMapper;
import com.huimv.farm.musk.mapper.DevEnergyMapper;
import com.huimv.farm.musk.mapper.DevEnvMapper;
import com.huimv.farm.musk.service.IDevEnergyService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author author
 * @since 2024-09-02
 */
@Service
public class DevEnergyServiceImpl extends ServiceImpl<DevEnergyMapper, DevEnergy> implements IDevEnergyService {

    @Autowired
    private DevEnergyMapper energyMapper;

    @Autowired
    private BaseBuildingMapper baseBuildingMapper;

    @Override
    public Result pageEnergy(Map<String, Object> map) {

        Integer current = (Integer) map.get("current");
        Integer size = (Integer) map.get(" ");
        String farmId = (String) map.get("farmId");
        String unitId = (String) map.get("unitId");

        IPage<DevEnergy> list = page(new Page<>(current, size), new QueryWrapper<DevEnergy>().eq(StringUtils.isNotBlank(unitId), "unit_id", unitId).eq("farm_id", farmId));

        return new Result(ResultCode.SUCCESS, list);
    }

    @Override
    public Result listEnergy(Map<String, String> map) {
        String startDate = map.get("startDate");
        String endDate = map.get("endDate");
        String unitId = map.get("unitId");
        return new Result(ResultCode.SUCCESS, this.list(new QueryWrapper<DevEnergy>().eq(StringUtils.isNotBlank(unitId), "unit_id", unitId)
                .ge(StringUtils.isNotBlank(startDate), "date_time", startDate).le(StringUtils.isNotBlank(endDate), "date_time", endDate)));

    }

    @Override
    public Result listHistory(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        String unitId = paramsMap.get("unitId");
        String timeType = paramsMap.get("timeType");

        if ("".equals(timeType) || null == timeType) {
            timeType = "1";
        }
        QueryWrapper<DevEnergy> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("unit_id", unitId);
        //今日
        List<DevEnergy> protEnvData = null;
        if ("1".equals(timeType)) {
            Date timesmorning = DataUill.getTimesmorning();
            queryWrapper.ge("date_time", timesmorning);
            protEnvData = energyMapper.listDataDay(queryWrapper);
        }
        //本周
        else if ("2".equals(timeType)) {
            DateTime dateTime = DateUtil.beginOfWeek(new Date());
            queryWrapper.ge("date_time", dateTime);
            protEnvData = energyMapper.listData(queryWrapper);
        }
        //本月
        else if ("3".equals(timeType)) {
            Date monthmorning = DataUill.getTimesMonthmorning();
            queryWrapper.ge("date_time", monthmorning);
            protEnvData = energyMapper.listData(queryWrapper);
        }
        //自定义
        else if ("4".equals(timeType)) {
            String startDate = paramsMap.get("startDate");
            String endDate = paramsMap.get("endDate");
            queryWrapper.between("date_time", startDate, endDate);
            protEnvData = energyMapper.listData(queryWrapper);
        }
        return new Result(ResultCode.SUCCESS, protEnvData);
    }

    @Override
    public Result listScreen(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        QueryWrapper<BaseBuilding> buildingQueryWrapper = new QueryWrapper<>();
        buildingQueryWrapper.eq("farm_id", farmId).eq("f_type", 1);
        List<BaseBuilding> buildings = baseBuildingMapper.selectList(buildingQueryWrapper);
        JSONArray jsonArray = new JSONArray();
        if (buildings.size() != 0) {
            for (BaseBuilding building : buildings) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("unitName", building.getBuildName());
                jsonObject.put("unitId", building.getId());
                QueryWrapper<DevEnergy> devEnvQueryWrapper = new QueryWrapper<>();
                devEnvQueryWrapper.eq("farm_id", farmId).orderByDesc("id").last(" limit 1").eq("unit_id", building.getId());
                DevEnergy devEnv = energyMapper.selectOne(devEnvQueryWrapper);
                if (ObjectUtil.isNotEmpty(devEnv)) {
                    jsonObject.put("water", devEnv.getWater());
                    jsonObject.put("elec", devEnv.getElec());
                } else {
                    jsonObject.put("water", "--");
                    jsonObject.put("elec", "--");
                }
                jsonArray.add(jsonObject);
            }
        }
        return new Result(ResultCode.SUCCESS, jsonArray);
    }

    @Override
    public Result listWeek(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.DATE, calendar.get(Calendar.DATE) - 7);
        QueryWrapper<DevEnergy> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).ge("date_time", calendar.getTime());
        List<DevEnergy> energies = energyMapper.listScreen(queryWrapper);
        return new Result(ResultCode.SUCCESS, energies);
    }

    @Override
    public Result listRank(Map<String, String> paramsMap) {
        String farmId = paramsMap.get("farmId");

        JSONArray jsonArray = new JSONArray();
        QueryWrapper<BaseBuilding> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("farm_id", farmId).eq("f_type", 1);
        List<BaseBuilding> buildings = baseBuildingMapper.selectList(queryWrapper);//得到所有的楼层
        Date monthmorning = DataUill.getTimesMonthmorning();
        for (BaseBuilding pigpen : buildings) {
            Float value = 0.00f;
            JSONObject jsonObject = new JSONObject();

            QueryWrapper<DevEnergy> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("farm_id", farmId).ge("date_time",monthmorning)
                    .eq("unit_id",pigpen.getId()).select(" IFNULL(SUM(water),0) water,IFNULL(SUM(elec),0) elec ");
            DevEnergy devEnergy = energyMapper.selectOne(queryWrapper2);
            jsonObject.put("water", devEnergy.getWater());
            jsonObject.put("elec", devEnergy.getElec());

            jsonObject.put("buildName", pigpen.getBuildName());
            jsonArray.add(jsonObject);
        }

        JSONArray arraySort = jsonArraySort(jsonArray, "water", true);
        return new Result(ResultCode.SUCCESS, arraySort);

    }

    /*数组排序*/
    public static JSONArray jsonArraySort(JSONArray jsonArr, String sortKey, boolean is_desc) {
        //存放排序结果json数组
        JSONArray sortedJsonArray = new JSONArray();
        //用于排序的list
        List<JSONObject> jsonValues = new ArrayList<JSONObject>();
        //将参数json数组每一项取出，放入list
        if (jsonArr.size() < 4) {
            for (int i = 0; i < jsonArr.size(); i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < jsonArr.size(); i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        } else {
            for (int i = 0; i < jsonArr.size(); i++) {
                jsonValues.add(jsonArr.getJSONObject(i));
            }
            //快速排序，重写compare方法，完成按指定字段比较，完成排序
            Collections.sort(jsonValues, new Comparator<JSONObject>() {
                //排序字段
                private final String KEY_NAME = sortKey;

                //重写compare方法
                @Override
                public int compare(JSONObject a, JSONObject b) {
                    //如果用String接会导致一位数和两位数混合比对的时候不能准确比出来，要用int类型接
                    //String valA = new String();
                    //String valB = new String();
                    Float valA = 0.00f;
                    Float valB = 0.00f;
                    try {
                        valA = Float.parseFloat(a.getString(KEY_NAME));
                        valB = Float.parseFloat(b.getString(KEY_NAME));
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                    //是升序还是降序
                    if (is_desc) {
                        return -valA.compareTo(valB);
                    } else {
                        return -valB.compareTo(valA);
                    }

                }
            });
            //将排序后结果放入结果jsonArray
            for (int i = 0; i < 4; i++) {
                sortedJsonArray.add(jsonValues.get(i));
            }
        }

        return sortedJsonArray;
    }
}
