package com.huimv.farm.musk.timer;


import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.huimv.farm.musk.breed.entity.BreedMuskInfo;
import com.huimv.farm.musk.breed.entity.MuskEarmark;
import com.huimv.farm.musk.breed.service.IBreedMuskInfoService;
import com.huimv.farm.musk.breed.service.IMuskEarmarkService;
import com.huimv.farm.musk.common.utils.NumberUtils;
import com.huimv.farm.musk.health.entity.HealthData;
import com.huimv.farm.musk.health.entity.HealthThreshold;
import com.huimv.farm.musk.health.entity.HealthWarning;
import com.huimv.farm.musk.health.service.IHealthDataService;
import com.huimv.farm.musk.health.service.IHealthThresholdService;
import com.huimv.farm.musk.health.service.IHealthWarningService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
@EnableScheduling
public class HealthTimer {

    @Autowired
    private IHealthDataService dataService;
    @Autowired
    private IHealthThresholdService thresholdService;
    @Autowired
    private IHealthWarningService warningService;
    @Autowired
    private IMuskEarmarkService muskEarmarkService;
    @Autowired
    private IBreedMuskInfoService infoService;

    @Scheduled(cron = "0 0 * * * ?")
    public void healthTimer() {
        HealthThreshold threshold = thresholdService.getOne(new QueryWrapper<HealthThreshold>().eq("farm_id", 330110007));
        List<MuskEarmark> earmarks = muskEarmarkService.list(new QueryWrapper<MuskEarmark>().eq("farm_id", 330110007)
                .eq("is_use",1).eq("status", 1));
        for (MuskEarmark earmark : earmarks) {
            String temp = NumberUtils.getNumFloat(37.0, 40.0);
            String act = NumberUtils.getNum(0, 400, 0);
            if (Double.valueOf(temp) > Double.valueOf(threshold.getTempUpper())) {
                HealthWarning warning = new HealthWarning();
                warning.setCreateTime(new Date());
                warning.setMuskId(earmark.getEarmark());
                warning.setFarmId(330110007);
                warning.setWarningType(1);
                warning.setContent(earmark.getEarmark()+"当前体温为"+temp+",体温过高");
                warningService.save(warning);
            }
            if (Double.valueOf(temp) < Double.valueOf(threshold.getTempDown())) {
                HealthWarning warning = new HealthWarning();
                warning.setCreateTime(new Date());
                warning.setMuskId(earmark.getEarmark());
                warning.setFarmId(330110007);
                warning.setWarningType(2);
                warning.setContent(earmark.getEarmark()+"当前体温为"+temp+",体温过低");
                warningService.save(warning);
            }
            if (Double.valueOf(act) > Double.valueOf(threshold.getActUpper())) {
                HealthWarning warning = new HealthWarning();
                warning.setCreateTime(new Date());
                warning.setMuskId(earmark.getEarmark());
                warning.setFarmId(330110007);
                warning.setWarningType(3);
                warning.setContent(earmark.getEarmark()+"当前运动量为"+act+",运动量过大");
                warningService.save(warning);
            }
            if (Double.valueOf(act) < Double.valueOf(threshold.getActDown())) {
                HealthWarning warning = new HealthWarning();
                warning.setCreateTime(new Date());
                warning.setMuskId(earmark.getEarmark());
                warning.setFarmId(330110007);
                warning.setWarningType(4);
                warning.setContent(earmark.getEarmark()+"当前运动量为"+act+",运动量过小");
                warningService.save(warning);
            }

            HealthData data = new HealthData();
            data.setAct(act);
            data.setTemp(temp);
            data.setFarmId(330110007);
            data.setCreateTime(new Date());
            data.setEarmark(earmark.getEarmark());
            dataService.save(data);

            earmark.setAct(act);
            earmark.setTemp(temp);
            earmark.setUpdateTime(new Date());
            muskEarmarkService.updateById(earmark);

            BreedMuskInfo muskInfo = infoService.getOne(new QueryWrapper<BreedMuskInfo>().eq("farm_id", 330110007)
                    .eq("musk_id", earmark.getEarmark()).eq("is_leave", 0));
            if (ObjectUtil.isNotEmpty(muskInfo)) {
                muskInfo.setAct(Integer.parseInt(act));
                muskInfo.setTemp(temp);
                infoService.updateById(muskInfo);
            }
        }
    }
}
